/*
 * Copyright 2020 ETSI
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holder nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software without 
 *    specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package fr.emse.gitlab.saref.checkers;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.nio.charset.Charset;
import java.nio.charset.MalformedInputException;

import org.apache.jena.atlas.RuntimeIOException;
import org.apache.jena.riot.Lang;

import fr.emse.gitlab.saref.SAREF;
import fr.emse.gitlab.saref.SAREFPipelineException;
import fr.emse.gitlab.saref.managers.RepositoryManager;
import fr.emse.gitlab.saref.utils.Languages;

/**
 * Checks TS 103 673 Clause 9.4.1: The ontology directory
 * 
 */
public class Clause_9_4_1_Checker extends AbstractClauseChecker {

	private static enum MESSAGE {
		one, name, turtle;
	}

	public Clause_9_4_1_Checker(RepositoryManager repositoryManager) {
		super(repositoryManager, Clause_9_4_1_Checker.class);
	}

	@Override
	public void checkClause() throws SAREFPipelineException {
		File dir = new File(repository.getDirectory(), "ontology");
		if (dir.list().length != 1) {
			logError(getMessage(MESSAGE.one, repository.getProject().getOntologyFileName(Languages.TEXT_TURTLE)));
		}
		File file = new File(dir, repository.getProject().getOntologyFileName(Languages.TEXT_TURTLE));
		if (!file.isFile()) {
			String msg = getMessage(MESSAGE.name, repository.getOntologyFileName(Languages.TEXT_TURTLE));
			logError(msg);
			throw new SAREFPipelineException(msg);
		}
		try (FileInputStream input = new FileInputStream(file)) {
			version.getModel().read(input, SAREF.BASE, Lang.TTL.getLabel());
		} catch (Exception ex) {
			if (ex instanceof RuntimeIOException && ex.getCause() instanceof MalformedInputException) {
				try (InputStreamReader input2 = new InputStreamReader(new FileInputStream(file),
						Charset.defaultCharset())) {
					version.getModel().read(input2, SAREF.BASE, Lang.TTL.getLabel());
				} catch (Exception ex2) {
					String msg = getMessage(MESSAGE.turtle);
					logError(msg, ex2);
					throw new SAREFPipelineException(msg, ex2);
				}
			} else {
				String msg = getMessage(MESSAGE.turtle);
				logError(msg, ex);
				throw new SAREFPipelineException(msg, ex);
			}
		}
	}

}
